//#######################################################################################################
//##                    This Plugin is only for use with the RFLink software package                   ##
//##                                      Plugin-064 PIR Xcellent Global                               ##
//#######################################################################################################
/*********************************************************************************************\
 * Ce protocole est pour les détecteurs infrarouge Xcellent Global
 * Note : Ceux-ci utilise le rolling code. On ne détecte donc que le nombre d'impulsions
 *
 * Author             : Prin Jean-Luc
 * License            : This code is free for use in any open source project when this header is included.
 *                      Usage of any parts of this code in a commercial application is prohibited!
 *********************************************************************************************
 * Données technique:
 * 5 fois 511 pulsations et longueur dernière pulsation = 210 ou 840
 *
 * Sample:
 * 20;03;DEBUG;Pulses=511;Pulses(uSec)=1725,325,1650,325,1625,350,575,1400,1600,400,1525,475,450,1550,1475,500,1500,500,450,1475,550,1450,
 * 550,1425,1600,400,525,1475,1500,450,1500,525,1475,500,425,1525,475,1475,500,1450,525,1450,1575,400,1550,425,1500,500,400,210,840;
 \*********************************************************************************************/

#ifdef PLUGIN_064

#define INTER_PULSATION                 1000            // Maximum de temps entre deux crénaux d'impulsions (en milisecondes).
unsigned long RecepMillis = 0;                          // Temps où est reçu le crénaux de 511 impulsions
unsigned long PreceRecepMillis = 0;                     // Temps où est reçu le précédent crénaux de 511 impulsions
byte NbreImpuls = 0;                                    // Nombre de crénaux d'impulsions dans les bons crénaux de temps
int DureePulse1;
int DureePulse2;

boolean Plugin_064(byte function, char *string) {

		if (RawSignal.Number == 511) {
                DureePulse1 = RawSignal.Pulses[510]*RAWSIGNAL_SAMPLE_RATE;
                DureePulse2 = RawSignal.Pulses[509]*RAWSIGNAL_SAMPLE_RATE;
            if ((DureePulse1 == 210) || (DureePulse1 == 840) && (DureePulse2 == 210)) {
                RecepMillis = millis();
                    if (PreceRecepMillis == 0) {
                        PreceRecepMillis = RecepMillis;
                    }
                Ecart = RecepMillis - PreceRecepMillis;
                // Serial.print(F("Ecart1 = "));
                // Serial.println(Ecart);
                    if (Ecart < INTER_PULSATION) {
                        NbreImpuls ++;
                        PreceRecepMillis = RecepMillis;
                        RecepMillis = millis();
                    }
                    else {
                        NbreImpuls = 1;
                        PreceRecepMillis = RecepMillis;
                        RecepMillis = millis();
                    }
                // Serial.print(F("Nbre1 = "));
                // Serial.println(NbreImpuls);
            }
        }

        if (NbreImpuls > 5) {
            //==================================================================================
			// Validity checks
			//==================================================================================
			// Output
			// ----------------------------------
			sprintf(pbuffer, "20;%02X;", PKSequenceNumber++); // Node and packet number
			Serial.print( pbuffer );
			// ----------------------------------
			Serial.print(F("Xcellent;ID=aaaaaa;SWITCH=01;CMD=ON"));
			Serial.println();
			//==================================================================================
			RecepMillis = 0;
			PreceRecepMillis = 0;
			NbreImpuls = 0;
			Ecart = 0;
			RawSignal.Repeats=true;                       // suppress repeats of the same RF packet
			RawSignal.Number=0;
			return true;
		}
}
#endif // Plugin_064
